// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <aoc/framework/Solver.hpp>

#include <algorithm>
#include <sstream>

template <typename T1, typename T2>
Solver<T1, T2>::Solver(const int inputFileNameSuffix)
    : inputFileNameSuffix_{ inputFileNameSuffix }, part1{}, part2{}
{
}

template <typename T1, typename T2>
const std::string Solver<T1, T2>::getInputFileName() const
{
    std::ostringstream stream;
    stream << clean(getPuzzleName());
    if (inputFileNameSuffix_ > 0)
    {
        stream << "_" << inputFileNameSuffix_;
    }
    stream << ".txt";
    return stream.str();
}

template <typename T1, typename T2>
const T1& Solver<T1, T2>::getResultPart1() const
{
    return part1;
}

template <typename T1, typename T2>
const T2& Solver<T1, T2>::getResultPart2() const
{
    return part2;
}

template <typename T1, typename T2>
std::string Solver<T1, T2>::clean(const std::string& original) const
{
    std::string cleaned{ original };
    std::transform(cleaned.begin(), cleaned.end(), cleaned.begin(),
        [](unsigned char c) { return c == ' ' ? '_' : std::tolower(c); });
    return cleaned;
}
