// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/CodeChronicle.hpp>

const std::string CodeChronicle::getPuzzleName() const
{
    return "Code Chronicle";
}

const int CodeChronicle::getPuzzleDay() const
{
    return 25;
}

void CodeChronicle::processDataLine(const std::string& line)
{
    if (line.empty())
    {
        finishCurrentSchematic();
        isNewSchematic_ = true;
    }
    else if (isNewSchematic_)
    {
        isNewSchematic_ = false;
        isLock_ = (line[0] == getFilledChar());
        resetCurrentSchematic();
    }
    else
    {
        updateCurrentSchematic(line);
    }
}

void CodeChronicle::finish()
{
    finishCurrentSchematic();
}

constexpr char CodeChronicle::getFilledChar()
{
    return '#';
}

constexpr unsigned CodeChronicle::getSchematicHeight()
{
    return 5;
}

void CodeChronicle::resetCurrentSchematic()
{
    current_ = &(isLock_ ? locks_ : keys_).emplace_back();
    // Sets the current schema to all -1's for keys, to compensate for counting their invariable "#" base line later.
    current_->fill(isLock_ ? 0 : -1);
}

void CodeChronicle::updateCurrentSchematic(const std::string& line)
{
    for (size_t i = 0; i < line.size(); i++)
    {
        if (line[i] == getFilledChar())
        {
            (*current_)[i]++;
        }
    }
}

void CodeChronicle::finishCurrentSchematic()
{
    for (auto& schema : isLock_ ? keys_ : locks_)
    {
        if (canFit(*current_, schema))
        {
            part1++;
        }
    }
}

bool CodeChronicle::canFit(const Schematic& schematic1, const Schematic& schematic2)
{
    for (size_t i = 0; i < schematic1.size(); i++)
    {
        if (schematic1[i] + schematic2[i] > getSchematicHeight())
        {
            return false;
        }
    }
    return true;
}
